#!/usr/bin/perl

use Moose;
use Coligacao;
use Partido;
use Vereador;

my @coligacoes;
my @partidos;
my @vereadores;

sub existePartido{ # Se existe o partido no array @partidos.
	my $p;
	if(@partidos == 0){ # Caso o array @partidos esteja vazio.
		return 0;
	}
	foreach $p (@partidos){
		if($p->get_NomePartido eq $_[0]){
			return $p;
		}
	}
	return 0;
}

sub existeColigacao{ # Se existe a coligação no array @coligacoes.
	my $c;
	if(@coligacoes == 0){ # Caso o array @coligacoes esteja vazio.
		return 0;
	}
	foreach $c (@coligacoes){
		if($c->get_NomeColigacao eq $_[0]){
			return $c;
		}
	}
	return 0;
}

sub updatePartido{ # Atualiza o partido.
	my $p;
	foreach $p (@partidos){
		if($p->get_NomePartido eq $_[0]){
			$p->incrementaVoto($_[1]);
			if($_[2] == 1){ # Se o candidato foi eleito.
				$p->incrementaCandidato;
			}
		}
	}
}

sub updateColigacao{ # Atualiza a coligação.
	my $c;
	foreach $c (@coligacoes){
		if($c->get_NomeColigacao eq $_[0]){
			$c->incrementaVoto($_[1]);
			if($_[2] == 1){ # Se o candidato foi eleito.
				$c->incrementaCandidato;
			}			
		}
	}
}

sub ordenaPartidosPorVoto{ # Ordena @partidos por voto.
	return sort {$b->get_VotoPartido <=> $a->get_VotoPartido} @partidos;
}

sub ordenaColigacoesPorVoto{ # Ordena @coligacoes por voto.
	return sort {$b->get_VotoColigacao <=> $a->get_VotoColigacao} @coligacoes;	
}

sub relatorioVotacaoPartidos{ # Imprime o relatório de votação dos partidos.
	my @partidosOrdenados = ordenaPartidosPorVoto;
	my $p;
	my $i = 1;
	print "Votação (nominal) dos partidos e número de candidatos eleitos:\n";
	foreach $p (@partidosOrdenados){
		print $i . " - " . $p->get_NomePartido. ", " . $p->get_VotoPartido;
		if($p->get_VotoPartido == 1 || $p->get_VotoPartido == 0){
			print " voto, ";
		}else{
			print " votos, ";
		}
		print $p->get_QCandPartido;
		if($p->get_QCandPartido == 1 || $p->get_QCandPartido == 0){
			print " candidato eleito\n";
		}else{
			print " candidatos eleitos\n";
		}
		$i++;
	}
	print "\n";
}

sub relatorioVotacaoColigacoes{ # Imprime o relatório de votação das coligações.
	my @coligacoesOrdenadas = ordenaColigacoesPorVoto;
	my $c;
	my $i = 1;
	print "Votação (nominal) das coligações e número de candidatos eleitos:\n";
	foreach $c (@coligacoesOrdenadas){
		print $i . " - Coligação: " . $c->get_NomeColigacao. ", " . $c->get_VotoColigacao;
		if($c->get_VotoColigacao == 1 || $c->get_VotoColigacao == 0){
			print " voto, ";
		}else{
			print " votos, ";
		}
		print $c->get_QCandColigacao;
		if($c->get_QCandColigacao == 1 || $c->get_QCandColigacao == 0){
			print " candidato eleito\n";
		}else{
			print " candidatos eleitos\n";
		}
		$i++;
	}
	print "\n";
}

sub relatorioVereadoresEleitos{ # Imprime o relatório de vereadores eleitos.
	my @coligacoesOrdenadas = ordenaColigacoesPorVoto;
	my $v;
	my $i = 1;
	print "Vereadores eleitos:\n";
	foreach $v (@vereadores){
		if ($v->get_Eleito){
			print $i . " - " . $v->get_NomeVereador . " (" . $v->get_Partido->get_NomePartido . ", " . $v->get_VotoVereador;
			if($v->get_VotoVereador == 1 || $v->get_VotoVereador == 0){
				print " voto)";
			}else{
				print " votos)";
			}
			print " - Coligação: " . $v->get_Partido->get_Coligacao->get_NomeColigacao unless !defined $v->get_Partido->get_Coligacao;
			print "\n";
			$i++;			
		}else{
			last;
		}
	}
	print "\n";
}

sub addAll{ # Cadastra os vereadores, partidos e coligações.
	my $v;
	my $p;
	my $c;
	if(@_ == 6){ # Se a lista de parâmetros é igual a 6 (Se existe coligação).		
		#Cria um novo vereador.
		$v = new Vereador($_[2], $_[0], $_[5]);

		#Verifica se existe partido no array.
		if(($p = existePartido($_[3])) == 0){
			#Cria um novo partido.
			$p = new Partido($_[3], $_[5], $_[0]);
			$v->set_Partido($p);

			# Adiciona o partido ao array @partidos.
			push @partidos, $p;
		}else{
			$v->set_Partido($p);
			#Se existir, apenas fará a atualização de votos e candidatos totais.
			updatePartido($_[3], $_[5], $_[0]);
		}

		#Verifica se existe coligação no array.
		if(($c = existeColigacao($_[4])) == 0){

			#Cria uma nova colicação.
			$c = new Coligacao($_[4], $_[5], $_[0]);
			$p->set_Coligacao($c);

			# Adiciona a coligação ao array @coligacoes.
			push @coligacoes, $c;
		}else{
			$p->set_Coligacao($c);
			#Se existir, apenas fará a atualização de votos e candidatos totais.
			updateColigacao($_[4], $_[5], $_[0]);
		}

	}elsif (@_ == 5){ # Se a lista de parâmetros é igual a 5 (Se não existe coligação).

		#Cria um novo vereador.
		$v = new Vereador($_[2], $_[0], $_[4]);

		#Verifica se existe partido no array.
		if(($p = existePartido($_[3])) == 0){

			#Cria um novo partido.
			$p = Partido->new($_[3], $_[4], $_[0]);
			$v->set_Partido($p);

			# Adiciona o partido ao array @partidos.
			push @partidos, $p;
		}else{
			$v->set_Partido($p);
			#Se existir, apenas fará a atualização de votos e candidatos totais.
			updatePartido($_[3], $_[4], $_[0]);
		}

		# #Verifica se existe coligação no array.
		if(!existeColigacao($_[3])){

			#Cria uma nova coligação.
			$c = new Coligacao($_[3], $_[4], $_[0]);

			# Adiciona a coligação ao array @coligacoes.
			push @coligacoes, $c;
		}else{

			#Se existir, apenas fará a atualização de votos e candidatos totais.
			updateColigacao($_[3], $_[4], $_[0]);
		}
	}else{
		warn "Parâmetros inválidos!";
	}

	# Adiciona o vereador ao array @vereadores.
	push @vereadores, $v;
	
}
1;